#include <stdlib.h>
#include <errno.h>

#include "stack.h"

struct stack
   {
   void ** data;
   size_t size;
   size_t max;
   };

////////////////////////////////////////////////////////////////////////////////
struct stack * stack_new(size_t const max)
   {
   if (max < 1)
      {
      errno = EINVAL;
      return NULL;
      }

   struct stack * s = malloc(sizeof(struct stack));
   if (NULL == s)
      return NULL;

   s->size = 0;
   s->max = max;

   s->data = calloc(s->max, sizeof(void *));
   if (NULL == s->data)
      {
      free(s);
      return NULL;
      }

   return s;
   }

////////////////////////////////////////////////////////////////////////////////
void * stack_push(struct stack * s, void * elem)
   {
   if ((NULL == s) || (NULL == elem))
      {
      errno = EINVAL;
      return NULL;
      }

   if (s->size == s->max)
      {
      errno = ENOMEM;
      return NULL;
      }

   return s->data[(s->size)++] = elem;
   }

////////////////////////////////////////////////////////////////////////////////
void * stack_pop(struct stack * s)
   {
   if (NULL == s)
      {
      errno = EINVAL;
      return NULL;
      }

   if (0 == s->size)
      {
      return NULL;
      }

   return s->data[--(s->size)];
   }

////////////////////////////////////////////////////////////////////////////////
size_t stack_size(struct stack * s)
   {
   if (NULL == s)
      {
      errno = EINVAL;
      return -1;
      }

   return s->size;
   }

////////////////////////////////////////////////////////////////////////////////
void stack_delete(struct stack * s)
   {
   if (NULL != s) free(s->data);
   free(s);
   }
